-- test the cpld 

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


entity test is
end entity;

architecture test_arch of test is
component system
  port (
    sys_clk : in std_logic;
    pp_dck : in std_logic;
    pp_dwr : in std_logic;
    pp_drd : out std_logic;
    pb_a : out std_logic_vector(0 to 17);
    pb_d : inout std_logic_vector(0 to 15);
    pb_oen : out std_logic;
    pb_wen : out std_logic;
    ram_cen : out std_logic;
    pb_ubn : out std_logic;
    pb_lbn : out std_logic
  );
end component;
component epp_model
	generic (T: time:= 41.6ns); -- CPLD clock perion = 24MHz
  port(pp_dck: out std_logic; -- data clock
  pp_drd: in std_logic; -- read data
	pp_dwr: out STD_LOGIC); -- write data
end component;

component sram_model
	generic (data_width: integer:= 16; -- width of data bus (must be 8, 16, 24, 32, ...)
	    adr_width: integer:= 4); -- width of the address bus
	port(oen : in std_logic; -- output enable (active low)
		  cen: in std_logic; -- chip enable
		  wen: in std_logic; -- write enable
      ben: in std_logic_vector(data_width/8 - 1 downto 0); -- byte enable active low (0-lower, 1-upper)
	  	adr: in std_logic_vector(adr_width-1 downto 0); -- address bus
   		data: inout std_logic_vector(0 to data_width-1)); -- bidirectional data bus
end component;

  constant sram_model_awidth: integer:= 10; -- SRAM model address width - less than acctual STAM awidth in order to reduce simulation time
-- PP FPGA signals
	signal clk: std_logic;
	signal pp_dck, pp_drd, pp_dwr : std_logic; -- parallel port (CPLD output) signals
-- SRAM signals
  signal pb_a : std_logic_vector(0 to 17);
  signal pb_d : std_logic_vector(0 to 15);
  signal ben: std_logic_vector(0 to 1); -- SRAM byte enable active low
  signal pb_oen, pb_wen, ram_cen, pb_ubn, pb_lbn: std_logic;

begin
UUT: system -- Unit under test
  port map(sys_clk=> clk, 
    -- epp signal
    pp_dck=> pp_dck, pp_drd=> pp_drd, pp_dwr=> pp_dwr,
    -- SRAM signals
    pb_a=> pb_a, pb_d=> pb_d, pb_oen=> pb_oen, 
    pb_wen=> pb_wen, ram_cen=> ram_cen, pb_ubn=> pb_ubn, pb_lbn=> pb_lbn);

    
em: epp_model
	generic map (T=> 41.6ns) -- CPLD clock perion = 24MHz
  port map (pp_dck=> pp_dck, pp_drd=> pp_drd, pp_dwr=> pp_dwr);

sm: sram_model
	generic map(data_width=> 16, adr_width=> sram_model_awidth)
	port map(oen=> pb_oen, cen=> ram_cen, wen=> pb_wen, ben=> ben, 
	  	adr=> pb_a(0 to sram_model_awidth-1), data=> pb_d);

ben<= pb_lbn & pb_ubn;

process begin
	  clk<= '1';
    wait for 10 ns;
	  clk<= '0';
	  wait for 10 ns;
 end process;
 

end test_arch;
