-- test the cpld 

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


entity test is
end entity;

architecture test_arch of test is
component system
  port (
    CLK : in std_logic;
    pp_dck : in std_logic;
    pp_dwr : in std_logic;
    pp_drd : out std_logic;
    pb_a : out std_logic_vector(0 to 17);
    pb_d : inout std_logic_vector(0 to 15);
    pb_oen : out std_logic;
    pb_wen : out std_logic;
    ram_cen : out std_logic;
    pb_ubn : out std_logic;
    pb_lbn : out std_logic
  );
end component;
component epp_model
	generic (T: time:= 41.6ns); -- CPLD clock perion = 24MHz
  port(pp_dck: out std_logic; -- data clock
  pp_drd: in std_logic; -- read data
	pp_dwr: out STD_LOGIC); -- write data
end component;
component sram_model
	generic (data_width: integer:= 16; -- width of data bus (must be 8, 16, 24, 32, ...)
	    adr_width: integer:= 4); -- width of the address bus
	port(oen : in std_logic; -- output enable (active low)
		  cen: in std_logic; -- chip enable
		  wen: in std_logic; -- write enable
      ben: in std_logic_vector(data_width/8 - 1 downto 0); -- byte enable active low (0-lower, 1-upper)
	  	adr: in std_logic_vector(adr_width-1 downto 0); -- address bus
   		data: inout std_logic_vector(0 to data_width-1)); -- bidirectional data bus
end component;


-- PP FPGA signals
	signal clk: std_logic;
	-- Parallel port signals
  signal pp_dck, pp_drd, pp_dwr : std_logic; -- parallel port (CPLD output) signals
  -- SRAM memory signals
  constant sram_awidth: integer:= 10; -- SRAM address width for simulation purposes (LSBs are used, MSBs are ignored)
  signal pb_a : std_logic_vector(0 to 17);
  signal pb_d : std_logic_vector(0 to 15);
  signal pb_oen, pb_wen, ram_cen: std_logic;
  signal pb_ben: std_logic_vector(0 to 1); -- byte enable signal
  

begin
UUT: system -- Unit under test
  port map(clk=> clk, 
    -- Parallel port connection
    pp_dck=> pp_dck, pp_drd=> pp_drd, pp_dwr=> pp_dwr,
    -- SRAM connection
    pb_a=> pb_a, pb_d=> pb_d, pb_oen=> pb_oen,
    pb_wen=> pb_wen, ram_cen=> ram_cen,
    pb_ubn=> pb_ben(1), pb_lbn=> pb_ben(0));

em: epp_model
	generic map (T=> 25ns) -- CPLD clock perion = 40MHz
  port map (pp_dck=> pp_dck, pp_drd=> pp_drd, pp_dwr=> pp_dwr);

 sm: sram_model
	generic map (data_width=> 16, adr_width=> sram_awidth) -- width of the address bus
	port map(oen=> pb_oen, cen=> ram_cen, wen=> pb_wen,
      ben=> pb_ben, adr=> pb_a(18-sram_awidth to 17),
   		data=>pb_d);
 
 process begin
	  clk<= '1';
    wait for 10 ns;
	  clk<= '0';
	  wait for 10 ns;
 end process;
 

end test_arch;
